from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.payment_method_card_checks import PaymentMethodCardChecks


class CardDetails(BaseModel):
    """The card details of a payment method."""

    brand: Optional[str] = None

    checks: PaymentMethodCardChecks = {}  # type: ignore

    country: Optional[str] = None

    exp_month: int = 0

    exp_year: int = 0

    fingerprint: Optional[str] = None

    funding: Optional[str] = None

    last4: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.error_code import ErrorCode


class ApiError(BaseModel):
    """An error."""

    error_code: ErrorCode

    message: str

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.jetstream_config import JetstreamConfig
from ..models.jetstream_stats import JetstreamStats
from ..models.meta_cluster_info import MetaClusterInfo


class Jetstream(BaseModel):
    """Jetstream information."""

    config: JetstreamConfig = {
        "domain": "",
        "max_memory": 0,
        "max_storage": 0,
        "store_dir": "",
    }  # type: ignore

    meta: MetaClusterInfo = {"cluster_size": 0, "leader": "", "name": ""}  # type: ignore

    stats: JetstreamStats = {
        "accounts": 0,
        "api": {"errors": 0, "inflight": 0, "total": 0},
        "ha_assets": 0,
        "memory": 0,
        "reserved_memory": 0,
        "reserved_store": 0,
        "store": 0,
    }  # type: ignore

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.camera_settings import CameraSettings


class CameraDragMove(BaseModel):
    """The response from the `CameraDragMove` command. Note this is an \"unreliable\" channel message, so this data may need more data like a \"sequence\" """

    settings: CameraSettings

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class PathComponentConstraintType(str, Enum):
    """The path component constraint type"""  # noqa: E501

    UNCONSTRAINED = "unconstrained"
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"
    EQUAL_LENGTH = "equal_length"
    PARALLEL = "parallel"
    ANGLE_BETWEEN = "angle_between"

    def __str__(self) -> str:
        return str(self.value)



from typing import List, Optional

from pydantic import BaseModel, ConfigDict

from ..models.side_face import SideFace


class ExtrudedFaceInfo(BaseModel):
    """IDs for the extruded faces."""

    bottom: Optional[str] = None

    sides: List[SideFace]

    top: str

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class AsyncApiCallType(str, Enum):
    """The type of async API call."""  # noqa: E501

    """# File conversion. """  # noqa: E501
    FILE_CONVERSION = "file_conversion"
    """# File volume. """  # noqa: E501
    FILE_VOLUME = "file_volume"
    """# File center of mass. """  # noqa: E501
    FILE_CENTER_OF_MASS = "file_center_of_mass"
    """# File mass. """  # noqa: E501
    FILE_MASS = "file_mass"
    """# File density. """  # noqa: E501
    FILE_DENSITY = "file_density"
    """# File surface area. """  # noqa: E501
    FILE_SURFACE_AREA = "file_surface_area"
    """# Text to CAD. """  # noqa: E501
    TEXT_TO_CAD = "text_to_cad"
    """# Text to CAD iteration. """  # noqa: E501
    TEXT_TO_CAD_ITERATION = "text_to_cad_iteration"
    """# Text to CAD multi-file iteration. """  # noqa: E501
    TEXT_TO_CAD_MULTI_FILE_ITERATION = "text_to_cad_multi_file_iteration"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.source_position import SourcePosition


class SourceRange(BaseModel):
    """A source range of code."""

    end: SourcePosition

    start: SourcePosition

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class PrivacySettings(BaseModel):
    """Privacy settings for an org or user."""

    can_train_on_data: bool

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class UnitPressure(str, Enum):
    """The valid types of pressure units."""  # noqa: E501

    """# Atmospheres <https://en.wikipedia.org/wiki/Standard_atmosphere_(unit)> """  # noqa: E501
    ATMOSPHERES = "atmospheres"
    """# Bars <https://en.wikipedia.org/wiki/Bar_(unit)> """  # noqa: E501
    BARS = "bars"
    """# Hectopascals <https://en.wikipedia.org/wiki/Hectopascal> """  # noqa: E501
    HECTOPASCALS = "hectopascals"
    """# Kilopascals <https://en.wikipedia.org/wiki/Kilopascal> """  # noqa: E501
    KILOPASCALS = "kilopascals"
    """# Millibars <https://en.wikipedia.org/wiki/Bar_(unit)> """  # noqa: E501
    MILLIBARS = "millibars"
    """# Pascals <https://en.wikipedia.org/wiki/Pascal_(unit)> """  # noqa: E501
    PASCALS = "pascals"
    """# Pounds per square inch (PSI) - <https://en.wikipedia.org/wiki/Pound_per_square_inch> """  # noqa: E501
    PSI = "psi"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.org_role import OrgRole
from ..models.uuid import Uuid


class OrgMember(BaseModel):
    """A member of an organization."""

    company: Optional[str] = None

    created_at: datetime.datetime

    discord: Optional[str] = None

    email: Optional[str] = None

    email_verified: Optional[datetime.datetime] = None

    first_name: Optional[str] = None

    github: Optional[str] = None

    id: Uuid

    image: str

    last_name: Optional[str] = None

    name: Optional[str] = None

    phone: str = ""

    role: OrgRole

    updated_at: datetime.datetime

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class JetstreamApiStats(BaseModel):
    """Jetstream API statistics."""

    errors: int = 0

    inflight: int = 0

    total: int = 0

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict


class ImportedGeometry(BaseModel):
    """Data from importing the files"""

    id: str

    value: List[str]

    model_config = ConfigDict(protected_namespaces=())



from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.global_axis import GlobalAxis


class OptionEuclidean(BaseModel):
    """Euclidean Distance."""

    type: Literal["euclidean"] = "euclidean"

    model_config = ConfigDict(protected_namespaces=())


class OptionOnAxis(BaseModel):
    """The distance between objects along the specified axis"""

    axis: GlobalAxis

    type: Literal["on_axis"] = "on_axis"

    model_config = ConfigDict(protected_namespaces=())


DistanceType = RootModel[
    Annotated[
        Union[
            OptionEuclidean,
            OptionOnAxis,
        ],
        Field(discriminator="type"),
    ]
]



"""Contains methods for accessing the shortlinks API paths: Shortlinks are a way to create a short URL that redirects to a longer URL. This is useful for sharing links that are long and unwieldy."""  # noqa: E501



from pydantic import BaseModel, ConfigDict

from ..models.point3d import Point3d


class CurveGetEndPoints(BaseModel):
    """Endpoints of a curve"""

    end: Point3d

    start: Point3d

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class JetstreamConfig(BaseModel):
    """Jetstream configuration."""

    domain: str = ""

    max_memory: int = 0

    max_storage: int = 0

    store_dir: str = ""

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class PlanInterval(str, Enum):
    """A plan's interval."""  # noqa: E501

    """# Day. """  # noqa: E501
    DAY = "day"
    """# Month. """  # noqa: E501
    MONTH = "month"
    """# Week. """  # noqa: E501
    WEEK = "week"
    """# Year. """  # noqa: E501
    YEAR = "year"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.point3d import Point3d
from ..models.unit_length import UnitLength


class CenterOfMass(BaseModel):
    """The center of mass response."""

    center_of_mass: Point3d

    output_unit: UnitLength

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class SelectWithPoint(BaseModel):
    """The response from the `SelectWithPoint` command."""

    entity_id: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Any

from pydantic import GetCoreSchemaHandler
from pydantic_core import CoreSchema, core_schema


class CountryCode(str):
    """An ISO-3166 alpha-2 country code. Always uppercase."""

    def __str__(self) -> str:
        return self

    @classmethod
    def __get_pydantic_core_schema__(
        cls, source_type: Any, handler: GetCoreSchemaHandler
    ) -> CoreSchema:
        return core_schema.no_info_after_validator_function(cls, handler(str))



from enum import Enum


class UnitAngle(str, Enum):
    """The valid types of angle formats."""  # noqa: E501

    """# Degrees <https://en.wikipedia.org/wiki/Degree_(angle)> """  # noqa: E501
    DEGREES = "degrees"
    """# Radians <https://en.wikipedia.org/wiki/Radian> """  # noqa: E501
    RADIANS = "radians"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict


class Solid3dGetOppositeEdge(BaseModel):
    """The response from the `Solid3dGetOppositeEdge` command."""

    edge: str

    model_config = ConfigDict(protected_namespaces=())



"""Contains methods for accessing the file API paths: CAD file operations. Create, get, and list CAD file conversions. More endpoints will be added here in the future as we build out transforms, etc on CAD models."""  # noqa: E501



from pydantic import BaseModel, ConfigDict


class GetNumObjects(BaseModel):
    """The response from the `GetNumObjects` command."""

    num_objects: int

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class SetCurrentToolProperties(BaseModel):
    """The response from the `SetCurrentToolProperties` endpoint."""

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class AnnotationLineEnd(str, Enum):
    """Annotation line end type"""  # noqa: E501

    NONE = "none"
    ARROW = "arrow"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ZooTool(str, Enum):
    """The Zoo tools that can make API calls."""  # noqa: E501

    """# The modeling app. """  # noqa: E501
    MODELING_APP = "modeling_app"
    """# The Text-to-CAD UI. """  # noqa: E501
    TEXT_TO_CAD = "text_to_cad"
    """# The Diff Chrome Extension. """  # noqa: E501
    DIFF_CHROME_EXTENSION = "diff_chrome_extension"

    def __str__(self) -> str:
        return str(self.value)



from typing import Union

from pydantic import BaseModel, ConfigDict, RootModel


class Response(BaseModel):
    """Response to the modeling command."""

    model_config = ConfigDict(protected_namespaces=())


class Errors(BaseModel):
    """Errors that occurred during the modeling command."""

    model_config = ConfigDict(protected_namespaces=())


BatchResponse = RootModel[
    Union[
        Response,
        Errors,
    ]
]



from typing import List

from pydantic import BaseModel, ConfigDict

from ..models.point3d import Point3d


class ProjectEntityToPlane(BaseModel):
    """The response from the `ProjectEntityToPlane` command."""

    projected_points: List[Point3d]

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.modeling_cmd import ModelingCmd
from ..models.modeling_cmd_id import ModelingCmdId


class ModelingCmdReq(BaseModel):
    """A graphics command submitted to the KittyCAD engine via the Modeling API."""

    cmd: ModelingCmd

    cmd_id: ModelingCmdId

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class CreatedAtSortMode(str, Enum):
    """Supported set of sort modes for scanning by created_at only.

    Currently, we only support scanning in ascending order."""  # noqa: E501

    """# Sort in increasing order of "created_at". """  # noqa: E501
    CREATED_AT_ASCENDING = "created_at_ascending"
    """# Sort in decreasing order of "created_at". """  # noqa: E501
    CREATED_AT_DESCENDING = "created_at_descending"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.modeling_app_subscription_tier import ModelingAppSubscriptionTier


class ZooProductSubscriptions(BaseModel):
    """A struct of Zoo product subscriptions."""

    modeling_app: ModelingAppSubscriptionTier

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class PathGetCurveUuid(BaseModel):
    """The response from the `PathGetCurveUuid` command."""

    curve_id: str

    model_config = ConfigDict(protected_namespaces=())



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict


class DiscountCode(BaseModel):
    """A discount code for a store."""

    code: str

    expires_at: Optional[datetime.datetime] = None

    percent_off: int

    model_config = ConfigDict(protected_namespaces=())



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict


class Onboarding(BaseModel):
    """Onboarding details"""

    first_call_from_modeling_app_date: Optional[datetime.datetime] = None

    first_call_from_text_to_cad_date: Optional[datetime.datetime] = None

    first_token_date: Optional[datetime.datetime] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict

from ..models.path_segment_info import PathSegmentInfo


class PathGetInfo(BaseModel):
    """The response from the `PathGetInfo` command."""

    segments: List[PathSegmentInfo]

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.payment_method import PaymentMethod
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/org/payment/methods".format(
        client.base_url,
    )  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[List[PaymentMethod], Error]]:
    if response.status_code == 200:
        response_200 = [PaymentMethod(**item) for item in response.json()]
        return response_200
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[List[PaymentMethod], Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
) -> Response[Optional[Union[List[PaymentMethod], Error]]]:
    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
) -> Optional[Union[List[PaymentMethod], Error]]:
    """This endpoint requires authentication by an org admin. It lists payment methods for the authenticated user's org."""  # noqa: E501

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
) -> Response[Optional[Union[List[PaymentMethod], Error]]]:
    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
) -> Optional[Union[List[PaymentMethod], Error]]:
    """This endpoint requires authentication by an org admin. It lists payment methods for the authenticated user's org."""  # noqa: E501

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



import datetime
from typing import Literal, Optional, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.fbx_storage import FbxStorage
from ..models.gltf_presentation import GltfPresentation
from ..models.gltf_storage import GltfStorage
from ..models.ply_storage import PlyStorage
from ..models.selection import Selection
from ..models.stl_storage import StlStorage
from ..models.system import System
from ..models.unit_length import UnitLength


class OptionFbx(BaseModel):
    """Autodesk Filmbox (FBX) format."""

    created: Optional[datetime.datetime] = None

    storage: FbxStorage

    type: Literal["fbx"] = "fbx"

    model_config = ConfigDict(protected_namespaces=())


class OptionGltf(BaseModel):
    """glTF 2.0. We refer to this as glTF since that is how our customers refer to it, although by default it will be in binary format and thus technically (glb). If you prefer ASCII output, you can set that option for the export."""

    presentation: GltfPresentation

    storage: GltfStorage

    type: Literal["gltf"] = "gltf"

    model_config = ConfigDict(protected_namespaces=())


class OptionObj(BaseModel):
    """Wavefront OBJ format."""

    coords: System

    type: Literal["obj"] = "obj"

    units: UnitLength

    model_config = ConfigDict(protected_namespaces=())


class OptionPly(BaseModel):
    """The PLY Polygon File Format."""

    coords: System

    selection: Selection

    storage: PlyStorage

    type: Literal["ply"] = "ply"

    units: UnitLength

    model_config = ConfigDict(protected_namespaces=())


class OptionStep(BaseModel):
    """ISO 10303-21 (STEP) format."""

    coords: System

    created: Optional[datetime.datetime] = None

    type: Literal["step"] = "step"

    model_config = ConfigDict(protected_namespaces=())


class OptionStl(BaseModel):
    """*ST**ereo**L**ithography format."""

    coords: System

    selection: Selection

    storage: StlStorage

    type: Literal["stl"] = "stl"

    units: UnitLength

    model_config = ConfigDict(protected_namespaces=())


OutputFormat3d = RootModel[
    Annotated[
        Union[
            OptionFbx,
            OptionGltf,
            OptionObj,
            OptionPly,
            OptionStep,
            OptionStl,
        ],
        Field(discriminator="type"),
    ]
]



from enum import Enum


class UnitFrequency(str, Enum):
    """The valid types of frequency units."""  # noqa: E501

    """# Gigahertz <https://en.wikipedia.org/wiki/Hertz> """  # noqa: E501
    GIGAHERTZ = "gigahertz"
    """# Hertz <https://en.wikipedia.org/wiki/Hertz> """  # noqa: E501
    HERTZ = "hertz"
    """# Kilohertz <https://en.wikipedia.org/wiki/Hertz> """  # noqa: E501
    KILOHERTZ = "kilohertz"
    """# Megahertz <https://en.wikipedia.org/wiki/Hertz> """  # noqa: E501
    MEGAHERTZ = "megahertz"
    """# Microhertz <https://en.wikipedia.org/wiki/Hertz> """  # noqa: E501
    MICROHERTZ = "microhertz"
    """# Millihertz <https://en.wikipedia.org/wiki/Hertz> """  # noqa: E501
    MILLIHERTZ = "millihertz"
    """# Nanohertz <https://en.wikipedia.org/wiki/Hertz> """  # noqa: E501
    NANOHERTZ = "nanohertz"
    """# Terahertz <https://en.wikipedia.org/wiki/Hertz> """  # noqa: E501
    TERAHERTZ = "terahertz"

    def __str__(self) -> str:
        return str(self.value)



from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.point3d import Point3d


class OptionLocal(BaseModel):
    """Local Origin (center of object bounding box)."""

    type: Literal["local"] = "local"

    model_config = ConfigDict(protected_namespaces=())


class OptionGlobal(BaseModel):
    """Global Origin (0, 0, 0)."""

    type: Literal["global"] = "global"

    model_config = ConfigDict(protected_namespaces=())


class OptionCustom(BaseModel):
    """Custom Origin (user specified point)."""

    origin: Point3d

    type: Literal["custom"] = "custom"

    model_config = ConfigDict(protected_namespaces=())


OriginType = RootModel[
    Annotated[
        Union[
            OptionLocal,
            OptionGlobal,
            OptionCustom,
        ],
        Field(discriminator="type"),
    ]
]



from enum import Enum


class ApiEndpoint(str, Enum):
    """Types of API endpoints."""  # noqa: E501

    """# The modeling API. """  # noqa: E501
    MODELING = "modeling"
    """# Machine learning API. """  # noqa: E501
    ML = "ml"
    """# File API. """  # noqa: E501
    FILE = "file"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.uuid import Uuid
from .base64data import Base64Data


class SamlIdentityProvider(BaseModel):
    """A SAML identity provider."""

    acs_url: str

    created_at: datetime.datetime

    id: Uuid

    idp_entity_id: Optional[str] = None

    idp_metadata_document_string: Optional[str] = None

    org_id: Uuid

    private_key: Optional[Base64Data] = None

    public_cert: Optional[Base64Data] = None

    slo_url: str

    technical_contact_email: Optional[str] = None

    updated_at: datetime.datetime

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class SupportTier(str, Enum):
    """The support tier the subscription provides."""  # noqa: E501

    """# Community support. """  # noqa: E501
    COMMUNITY = "community"
    """# Standard support. """  # noqa: E501
    STANDARD = "standard"
    """# Premium support. """  # noqa: E501
    PREMIUM = "premium"
    """# Priority support. """  # noqa: E501
    PRIORITY = "priority"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ImageFormat(str, Enum):
    """Enum containing the variety of image formats snapshots may be exported to."""  # noqa: E501

    """# .png format """  # noqa: E501
    PNG = "png"
    """# .jpeg format """  # noqa: E501
    JPEG = "jpeg"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.extended_user import ExtendedUser
from ...models.user_identifier import UserIdentifier
from ...types import Response


def _get_kwargs(
    id: UserIdentifier,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/users-extended/{id}".format(
        client.base_url,
        id=id,
    )  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[ExtendedUser, Error]]:
    if response.status_code == 200:
        response_200 = ExtendedUser(**response.json())
        return response_200
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[ExtendedUser, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: UserIdentifier,
    *,
    client: Client,
) -> Response[Optional[Union[ExtendedUser, Error]]]:
    kwargs = _get_kwargs(
        id=id,
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: UserIdentifier,
    *,
    client: Client,
) -> Optional[Union[ExtendedUser, Error]]:
    """To get information about yourself, use `/users-extended/me` as the endpoint. By doing so you will get the user information for the authenticated user.

    Alternatively, to get information about the authenticated user, use `/user/extended` endpoint.

    To get information about any Zoo user, you must be a Zoo employee."""  # noqa: E501

    return sync_detailed(
        id=id,
        client=client,
    ).parsed


async def asyncio_detailed(
    id: UserIdentifier,
    *,
    client: Client,
) -> Response[Optional[Union[ExtendedUser, Error]]]:
    kwargs = _get_kwargs(
        id=id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: UserIdentifier,
    *,
    client: Client,
) -> Optional[Union[ExtendedUser, Error]]:
    """To get information about yourself, use `/users-extended/me` as the endpoint. By doing so you will get the user information for the authenticated user.

    Alternatively, to get information about the authenticated user, use `/user/extended` endpoint.

    To get information about any Zoo user, you must be a Zoo employee."""  # noqa: E501

    return (
        await asyncio_detailed(
            id=id,
            client=client,
        )
    ).parsed



from enum import Enum


class AnnotationTextAlignmentX(str, Enum):
    """Horizontal Text alignment"""  # noqa: E501

    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.file_export_format import FileExportFormat
from ...models.text_to_cad import TextToCad
from ...models.text_to_cad_create_body import TextToCadCreateBody
from ...types import Response


def _get_kwargs(
    output_format: FileExportFormat,
    body: TextToCadCreateBody,
    *,
    client: Client,
    kcl: Optional[bool] = None,
) -> Dict[str, Any]:
    url = "{}/ai/text-to-cad/{output_format}".format(
        client.base_url,
        output_format=output_format,
    )  # noqa: E501

    if kcl is not None:
        if "?" in url:
            url = url + "&kcl=" + str(kcl)
        else:
            url = url + "?kcl=" + str(kcl)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "content": body.model_dump_json(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[TextToCad, Error]]:
    if response.status_code == 201:
        response_201 = TextToCad(**response.json())
        return response_201
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[TextToCad, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    output_format: FileExportFormat,
    body: TextToCadCreateBody,
    *,
    client: Client,
    kcl: Optional[bool] = None,
) -> Response[Optional[Union[TextToCad, Error]]]:
    kwargs = _get_kwargs(
        output_format=output_format,
        kcl=kcl,
        body=body,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    output_format: FileExportFormat,
    body: TextToCadCreateBody,
    *,
    client: Client,
    kcl: Optional[bool] = None,
) -> Optional[Union[TextToCad, Error]]:
    """Because our source of truth for the resulting model is a STEP file, you will always have STEP file contents when you list your generated models. Any other formats you request here will also be returned when you list your generated models.

    This operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint.

    One thing to note, if you hit the cache, this endpoint will return right away. So you only have to wait if the status is not `Completed` or `Failed`."""  # noqa: E501

    return sync_detailed(
        output_format=output_format,
        kcl=kcl,
        body=body,
        client=client,
    ).parsed


async def asyncio_detailed(
    output_format: FileExportFormat,
    body: TextToCadCreateBody,
    *,
    client: Client,
    kcl: Optional[bool] = None,
) -> Response[Optional[Union[TextToCad, Error]]]:
    kwargs = _get_kwargs(
        output_format=output_format,
        kcl=kcl,
        body=body,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    output_format: FileExportFormat,
    body: TextToCadCreateBody,
    *,
    client: Client,
    kcl: Optional[bool] = None,
) -> Optional[Union[TextToCad, Error]]:
    """Because our source of truth for the resulting model is a STEP file, you will always have STEP file contents when you list your generated models. Any other formats you request here will also be returned when you list your generated models.

    This operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint.

    One thing to note, if you hit the cache, this endpoint will return right away. So you only have to wait if the status is not `Completed` or `Failed`."""  # noqa: E501

    return (
        await asyncio_detailed(
            output_format=output_format,
            kcl=kcl,
            body=body,
            client=client,
        )
    ).parsed



from typing import Any, Dict

from websockets.asyncio.client import (
    ClientConnection as ClientConnectionAsync,
    connect as ws_connect_async,
)
from websockets.sync.client import (
    ClientConnection as ClientConnectionSync,
    connect as ws_connect,
)

from ...client import Client


def _get_kwargs(
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/ws/executor/term".format(client.base_url)  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def sync(
    *,
    client: Client,
) -> ClientConnectionSync:
    """Attach to a docker container to create an interactive terminal."""  # noqa: E501

    kwargs = _get_kwargs(
        client=client,
    )

    return ws_connect(
        kwargs["url"].replace("http", "ws"),
        additional_headers=kwargs["headers"],
        close_timeout=120,
        max_size=None,
    )  # type: ignore


async def asyncio(
    *,
    client: Client,
) -> ClientConnectionAsync:
    """Attach to a docker container to create an interactive terminal."""  # noqa: E501

    kwargs = _get_kwargs(
        client=client,
    )

    return await ws_connect_async(
        kwargs["url"].replace("http", "ws"),
        extra_headers=kwargs["headers"],
        close_timeout=120,
        max_size=None,
    )



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.extrusion_face_cap_type import ExtrusionFaceCapType


class ExtrusionFaceInfo(BaseModel):
    """Extrusion face info struct (useful for maintaining mappings between source path segment ids and extrusion faces)"""

    cap: ExtrusionFaceCapType

    curve_id: Optional[str] = None

    face_id: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class MlPromptType(str, Enum):
    """A type of ML prompt."""  # noqa: E501

    """# Text to CAD. """  # noqa: E501
    TEXT_TO_CAD = "text_to_cad"
    """# Text to KCL. """  # noqa: E501
    TEXT_TO_KCL = "text_to_kcl"
    """# Text to KCL iteration. """  # noqa: E501
    TEXT_TO_KCL_ITERATION = "text_to_kcl_iteration"
    """# Text to KCL iteration with multiple files. """  # noqa: E501
    TEXT_TO_KCL_MULTI_FILE_ITERATION = "text_to_kcl_multi_file_iteration"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class CutType(str, Enum):
    """What kind of cut to do"""  # noqa: E501

    """# Round off an edge. """  # noqa: E501
    FILLET = "fillet"
    """# Cut away an edge. """  # noqa: E501
    CHAMFER = "chamfer"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.api_call_status import ApiCallStatus
from ..models.file_import_format import FileImportFormat
from ..models.point3d import Point3d
from ..models.unit_length import UnitLength
from ..models.uuid import Uuid


class FileCenterOfMass(BaseModel):
    """A file center of mass result."""

    center_of_mass: Optional[Point3d] = None

    completed_at: Optional[datetime.datetime] = None

    created_at: datetime.datetime

    error: Optional[str] = None

    id: Uuid

    output_unit: UnitLength

    src_format: FileImportFormat

    started_at: Optional[datetime.datetime] = None

    status: ApiCallStatus

    updated_at: datetime.datetime

    user_id: Uuid

    model_config = ConfigDict(protected_namespaces=())



from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.system import System
from ..models.unit_length import UnitLength


class OptionFbx(BaseModel):
    """Autodesk Filmbox (FBX) format."""

    type: Literal["fbx"] = "fbx"

    model_config = ConfigDict(protected_namespaces=())


class OptionGltf(BaseModel):
    """Binary glTF 2.0. We refer to this as glTF since that is how our customers refer to it, but this can also import binary glTF (glb)."""

    type: Literal["gltf"] = "gltf"

    model_config = ConfigDict(protected_namespaces=())


class OptionObj(BaseModel):
    """Wavefront OBJ format."""

    coords: System

    type: Literal["obj"] = "obj"

    units: UnitLength

    model_config = ConfigDict(protected_namespaces=())


class OptionPly(BaseModel):
    """The PLY Polygon File Format."""

    coords: System

    type: Literal["ply"] = "ply"

    units: UnitLength

    model_config = ConfigDict(protected_namespaces=())


class OptionSldprt(BaseModel):
    """SolidWorks part (SLDPRT) format."""

    split_closed_faces: bool = False

    type: Literal["sldprt"] = "sldprt"

    model_config = ConfigDict(protected_namespaces=())


class OptionStep(BaseModel):
    """ISO 10303-21 (STEP) format."""

    split_closed_faces: bool = False

    type: Literal["step"] = "step"

    model_config = ConfigDict(protected_namespaces=())


class OptionStl(BaseModel):
    """*ST**ereo**L**ithography format."""

    coords: System

    type: Literal["stl"] = "stl"

    units: UnitLength

    model_config = ConfigDict(protected_namespaces=())


InputFormat3d = RootModel[
    Annotated[
        Union[
            OptionFbx,
            OptionGltf,
            OptionObj,
            OptionPly,
            OptionSldprt,
            OptionStep,
            OptionStl,
        ],
        Field(discriminator="type"),
    ]
]



from pydantic import BaseModel, ConfigDict

from ..models.length_unit import LengthUnit


class EntityGetDistance(BaseModel):
    """The response from the `EntitiesGetDistance` command."""

    max_distance: LengthUnit

    min_distance: LengthUnit

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.rtc_sdp_type import RtcSdpType


class RtcSessionDescription(BaseModel):
    """SessionDescription is used to expose local and remote session descriptions."""

    sdp: str

    type: RtcSdpType

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.unit_area import UnitArea


class SurfaceArea(BaseModel):
    """The surface area response."""

    output_unit: UnitArea

    surface_area: float

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class TextToCadModel(str, Enum):
    """A type of Text-to-CAD model."""  # noqa: E501

    """# CAD. """  # noqa: E501
    CAD = "cad"
    """# KCL. """  # noqa: E501
    KCL = "kcl"
    """# KCL iteration. """  # noqa: E501
    KCL_ITERATION = "kcl_iteration"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.curve_type import CurveType


class CurveGetType(BaseModel):
    """The response from the `CurveGetType` command."""

    curve_type: CurveType

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class RawFile(BaseModel):
    """A raw file with unencoded contents to be passed over binary websockets. When raw files come back for exports it is sent as binary/bson, not text/json."""

    contents: bytes

    name: str

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class UpdatePaymentBalance(BaseModel):
    """The data for updating a balance."""

    monthly_api_credits_remaining_monetary_value: Optional[float] = None

    stable_api_credits_remaining_monetary_value: Optional[float] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict

from ..models.point3d import Point3d


class ProjectPointsToPlane(BaseModel):
    """The response from the `ProjectPointsToPlane` command."""

    projected_points: List[Point3d]

    model_config = ConfigDict(protected_namespaces=())



from typing import List, Optional

from pydantic import BaseModel, ConfigDict

from ..models.output_file import OutputFile


class CodeOutput(BaseModel):
    """Output of the code being executed.

    <details><summary>JSON schema</summary>

    ```json { \"description\": \"Output of the code being executed.\", \"type\": \"object\", \"properties\": { \"output_files\": { \"description\": \"The contents of the files requested if they were passed.\", \"type\": \"array\", \"items\": { \"$ref\": \"#/components/schemas/OutputFile\" } }, \"stderr\": { \"description\": \"The stderr of the code.\", \"default\": \"\", \"type\": \"string\" }, \"stdout\": { \"description\": \"The stdout of the code.\", \"default\": \"\", \"type\": \"string\" } } } ``` </details>"""

    output_files: Optional[List[OutputFile]] = None

    stderr: str = ""

    stdout: str = ""

    model_config = ConfigDict(protected_namespaces=())



from typing import List, Optional

from pydantic import BaseModel, ConfigDict

from ..models.api_error import ApiError


class FailureWebSocketResponse(BaseModel):
    """Unsuccessful Websocket response."""

    errors: List[ApiError]

    request_id: Optional[str] = None

    success: bool

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class LeafNode(BaseModel):
    """Leaf node information."""

    auth_timeout: int = 0

    host: str = ""

    port: int = 0

    tls_timeout: int = 0

    model_config = ConfigDict(protected_namespaces=())



from typing import List, Optional

from pydantic import BaseModel, ConfigDict

from ..models.source_range_prompt import SourceRangePrompt


class TextToCadMultiFileIterationBody(BaseModel):
    """Body for iterating on models from text prompts."""

    kcl_version: Optional[str] = None

    project_name: Optional[str] = None

    prompt: Optional[str] = None

    source_ranges: Optional[List[SourceRangePrompt]] = None

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class DxfStorage(str, Enum):
    """Export storage."""  # noqa: E501

    """# Plaintext encoding.

This is the default setting. """  # noqa: E501
    ASCII = "ascii"
    """# Binary encoding. """  # noqa: E501
    BINARY = "binary"

    def __str__(self) -> str:
        return str(self.value)



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.point2d import Point2d


class PlaneIntersectAndProject(BaseModel):
    """Corresponding coordinates of given window coordinates, intersected on given plane."""

    plane_coordinates: Optional[Point2d] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated


class DxfData(BaseModel):
    """Export storage."""

    model_config = ConfigDict(protected_namespaces=())


class OptionDxf(BaseModel):
    """AutoCAD drawing interchange format."""

    storage: DxfData

    type: Literal["dxf"] = "dxf"

    model_config = ConfigDict(protected_namespaces=())


OutputFormat2d = RootModel[Annotated[Union[OptionDxf,], Field(discriminator="type")]]



import os
import ssl
from typing import Dict, Optional, Union

import attr

DEFAULT_BASE_URL = "https://api.zoo.dev"


@attr.s(auto_attribs=True)
class Client:
    """A Client which has been authenticated for use on secured endpoints of the KittyCAD API."""  # noqa: E501

    token: str = attr.ib(kw_only=True)
    base_url: str = attr.ib(default=DEFAULT_BASE_URL)
    cookies: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    headers: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    timeout: float = attr.ib(120.0, kw_only=True)
    verify_ssl: Union[str, bool, ssl.SSLContext] = attr.ib(True, kw_only=True)

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in all endpoints"""
        return {"Authorization": f"Bearer {self.token}", **self.headers}

    def with_headers(self, headers: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional headers"""
        return attr.evolve(self, headers={**self.headers, **headers})

    def get_cookies(self) -> Dict[str, str]:
        return {**self.cookies}

    def with_cookies(self, cookies: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional cookies"""
        return attr.evolve(self, cookies={**self.cookies, **cookies})

    def get_timeout(self) -> float:
        return self.timeout

    def with_timeout(self, timeout: float) -> "Client":
        """Get a new client matching this one with a new timeout (in seconds)"""
        return attr.evolve(self, timeout=timeout)

    def with_base_url(self, url: str) -> "Client":
        """Get a new client matching this one with a new base url"""
        return attr.evolve(self, base_url=url)


@attr.s(auto_attribs=True)
class ClientFromEnv(Client):
    """A Client which has been authenticated for use on secured endpoints that uses the KITTYCAD_API_TOKEN or ZOO_API_TOKEN environment variable for the authentication token.

    Optionally, you can use `ZOO_HOST` to set the base url.
    This implies you are hosting your own instance of the KittyCAD API.
    """  # noqa: E501

    token: str = attr.field()
    base_url: str = attr.ib(default=os.getenv("ZOO_HOST", DEFAULT_BASE_URL))

    @token.default
    def set_token(self):
        maybe_token: Optional[str] = os.getenv("KITTYCAD_API_TOKEN")
        if maybe_token is None:
            # Try the ZOO_API_TOKEN environment variable
            maybe_token = os.getenv("ZOO_API_TOKEN")
            if maybe_token is None:
                raise ValueError(
                    "KITTYCAD_API_TOKEN or ZOO_API_TOKEN environment variable must be set to use ClientFromEnv"
                )
        token: str = maybe_token

        return token

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in authenticated endpoints"""
        return {"Authorization": f"Bearer {self.token}", **self.headers}



from pydantic import BaseModel, ConfigDict


class SetObjectTransform(BaseModel):
    """The response from the `SetObjectTransform` command."""

    model_config = ConfigDict(protected_namespaces=())



import datetime
from typing import Dict, Optional

from pydantic import BaseModel, ConfigDict

from ..models.address_details import AddressDetails
from ..models.currency import Currency


class Customer(BaseModel):
    """The resource representing a payment \"Customer\"."""

    address: Optional[AddressDetails] = None

    balance: float = 0.0

    created_at: datetime.datetime

    currency: Currency = "usd"  # type: ignore

    delinquent: bool = False

    email: Optional[str] = None

    id: Optional[str] = None

    metadata: Dict[str, str] = {}

    name: Optional[str] = None

    phone: str = ""

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class ObjectBringToFront(BaseModel):
    """The response from the `ObjectBringToFront` endpoint."""

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.country_code import CountryCode


class AddressDetails(BaseModel):
    """Address details."""

    city: Optional[str] = None

    country: CountryCode

    state: Optional[str] = None

    street1: Optional[str] = None

    street2: Optional[str] = None

    zip: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import List, Optional

from pydantic import BaseModel, ConfigDict


class BooleanUnion(BaseModel):
    """The response from the 'BooleanUnion'."""

    extra_solid_ids: Optional[List[str]] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class TextToCadCreateBody(BaseModel):
    """Body for generating models from text."""

    kcl_version: Optional[str] = None

    project_name: Optional[str] = None

    prompt: str

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class ExtrusionFaceCapType(str, Enum):
    """Possible types of faces which can be extruded from a 3D solid."""  # noqa: E501

    """# Uncapped. """  # noqa: E501
    NONE = "none"
    """# Capped on top. """  # noqa: E501
    TOP = "top"
    """# Capped below. """  # noqa: E501
    BOTTOM = "bottom"
    """# Capped on both ends. """  # noqa: E501
    BOTH = "both"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.block_reason import BlockReason
from ..models.org_role import OrgRole
from ..models.uuid import Uuid


class UserOrgInfo(BaseModel):
    """A user's information about an org, including their role."""

    allow_users_in_domain_to_auto_join: Optional[bool] = None

    billing_email: Optional[str] = None

    billing_email_verified: Optional[datetime.datetime] = None

    block: Optional[BlockReason] = None

    created_at: datetime.datetime

    domain: Optional[str] = None

    id: Uuid

    image: Optional[str] = None

    name: Optional[str] = None

    phone: str = ""

    role: OrgRole

    stripe_id: Optional[str] = None

    updated_at: datetime.datetime

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.modeling_cmd_id import ModelingCmdId
from ..models.path_command import PathCommand


class PathSegmentInfo(BaseModel):
    """Info about a path segment"""

    command: PathCommand

    command_id: Optional[ModelingCmdId] = None

    relative: bool

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class PerspectiveCameraParameters(BaseModel):
    """Defines a perspective view."""

    fov_y: Optional[float] = None

    z_far: Optional[float] = None

    z_near: Optional[float] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.account_provider import AccountProvider
from ...models.error import Error
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/user/oauth2/providers".format(
        client.base_url,
    )  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[List[AccountProvider], Error]]:
    if response.status_code == 200:
        response_200 = [AccountProvider(**item) for item in response.json()]
        return response_200
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[List[AccountProvider], Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
) -> Response[Optional[Union[List[AccountProvider], Error]]]:
    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
) -> Optional[Union[List[AccountProvider], Error]]:
    """If this returns an empty array, then the user has not connected any OAuth2 providers and uses raw email authentication.

    This endpoint requires authentication by any Zoo user. It gets the providers for the authenticated user."""  # noqa: E501

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
) -> Response[Optional[Union[List[AccountProvider], Error]]]:
    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
) -> Optional[Union[List[AccountProvider], Error]]:
    """If this returns an empty array, then the user has not connected any OAuth2 providers and uses raw email authentication.

    This endpoint requires authentication by any Zoo user. It gets the providers for the authenticated user."""  # noqa: E501

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from pydantic import BaseModel, ConfigDict

from .base64data import Base64Data


class TakeSnapshot(BaseModel):
    """The response from the `TakeSnapshot` command."""

    contents: Base64Data

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class CreateShortlinkRequest(BaseModel):
    """Request to create a shortlink."""

    password: Optional[str] = None

    restrict_to_org: bool = False

    url: str

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class PathCommand(str, Enum):
    """The path component command type (within a Path)"""  # noqa: E501

    MOVE_TO = "move_to"
    LINE_TO = "line_to"
    BEZ_CURVE_TO = "bez_curve_to"
    NURBS_CURVE_TO = "nurbs_curve_to"
    ADD_ARC = "add_arc"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitTorque(str, Enum):
    """The valid types of torque units."""  # noqa: E501

    """# Newton metres <https://en.wikipedia.org/wiki/Newton_metre> """  # noqa: E501
    NEWTON_METRES = "newton_metres"
    """# Pound foot <https://en.wikipedia.org/wiki/Pound-foot_(torque)> """  # noqa: E501
    POUND_FOOT = "pound_foot"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.api_token_results_page import ApiTokenResultsPage
from ...models.created_at_sort_mode import CreatedAtSortMode
from ...models.error import Error
from ...types import Response


def _get_kwargs(
    sort_by: CreatedAtSortMode,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Dict[str, Any]:
    url = "{}/user/api-tokens".format(
        client.base_url,
    )  # noqa: E501

    if limit is not None:
        if "?" in url:
            url = url + "&limit=" + str(limit)
        else:
            url = url + "?limit=" + str(limit)

    if page_token is not None:
        if "?" in url:
            url = url + "&page_token=" + str(page_token)
        else:
            url = url + "?page_token=" + str(page_token)

    if sort_by is not None:
        if "?" in url:
            url = url + "&sort_by=" + str(sort_by)
        else:
            url = url + "?sort_by=" + str(sort_by)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[ApiTokenResultsPage, Error]]:
    if response.status_code == 200:
        response_200 = ApiTokenResultsPage(**response.json())
        return response_200
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[ApiTokenResultsPage, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    sort_by: CreatedAtSortMode,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Response[Optional[Union[ApiTokenResultsPage, Error]]]:
    kwargs = _get_kwargs(
        limit=limit,
        page_token=page_token,
        sort_by=sort_by,
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    sort_by: CreatedAtSortMode,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Optional[Union[ApiTokenResultsPage, Error]]:
    """This endpoint requires authentication by any Zoo user. It returns the API tokens for the authenticated user.

    The API tokens are returned in order of creation, with the most recently created API tokens first."""  # noqa: E501

    return sync_detailed(
        limit=limit,
        page_token=page_token,
        sort_by=sort_by,
        client=client,
    ).parsed


async def asyncio_detailed(
    sort_by: CreatedAtSortMode,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Response[Optional[Union[ApiTokenResultsPage, Error]]]:
    kwargs = _get_kwargs(
        limit=limit,
        page_token=page_token,
        sort_by=sort_by,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    sort_by: CreatedAtSortMode,
    *,
    client: Client,
    limit: Optional[int] = None,
    page_token: Optional[str] = None,
) -> Optional[Union[ApiTokenResultsPage, Error]]:
    """This endpoint requires authentication by any Zoo user. It returns the API tokens for the authenticated user.

    The API tokens are returned in order of creation, with the most recently created API tokens first."""  # noqa: E501

    return (
        await asyncio_detailed(
            limit=limit,
            page_token=page_token,
            sort_by=sort_by,
            client=client,
        )
    ).parsed
